package uk.ac.shef.is.imageClef.cluster;

import java.util.HashMap;

import uk.ac.shef.is.imageClef.clusterAssessment.ClusterTopicAssessmentsContainer;
import uk.ac.shef.is.imageClef.qrel.QrelsContainer;
import uk.ac.shef.is.imageClef.trecResult.TrecResultsContainer;

/** Class implements methods to get the cluster recall
 * @author Thomas Arni
 *
 */
public class ClusterRecall {
	
	private static boolean printDetails = false;
	
	/** Return the cluster recall from a certain topic at a certain recall position
	 * @param topicNr topic number
	 * @param clusterRecallAtPosition cluster recall position (e.g 20)
	 * @param trc TrecResultsContainer, which contains the results
	 * @param qrelsContainer QrelsContainer, which contains the Qrel file
	 * @param clusterAssessmentContainer Cluster assessment container
	 * @return cluster recall
	 */
	public static double getClusterRecall(String topicNr, int clusterRecallAtPosition, TrecResultsContainer trc, QrelsContainer qrelsContainer, HashMap<String, ClusterTopicAssessmentsContainer> clusterAssessmentContainer){
		double clusterRecall = 0.0d; //set 0.0 as default value
		String runName = Cluster.getRunName(trc.getFilePath());
		//Hash<clusterId, clusterCount>
		HashMap<String, Integer>  numberOfUniqueClusters = new HashMap<String, Integer>();
		if(trc.hasTopicResults(topicNr) && qrelsContainer.isValidTopicNr(topicNr)){
			for(int rank = 0; rank<clusterRecallAtPosition; rank++){
				String docNr = trc.getDocNrByTopicNrAndRank(topicNr, ""+rank);
				//only take the relevant ones
				if(qrelsContainer.isRelevant(topicNr, docNr)){
					ClusterTopicAssessmentsContainer ctac = clusterAssessmentContainer.get(topicNr); //contains all lines for one topic
					String clusterId = ctac.getClusterIdByDocNrAndTopicNr(docNr);
					if(printDetails) System.out.println("R:"+rank+" || DocNr:"+docNr + " || ClusterId:"+ clusterId + " || Topic:" + topicNr + " || RunName:"+runName);
					if(numberOfUniqueClusters.containsKey(clusterId)){
						int clusterCount = numberOfUniqueClusters.get(clusterId);
						numberOfUniqueClusters.put(clusterId, clusterCount++);
					}
					else numberOfUniqueClusters.put(clusterId, 1);
				}
			}
			double numberOfUniqueCoveredClusters = numberOfUniqueClusters.size(); //per topic
			double numberOfTotalClusters = clusterAssessmentContainer.get(topicNr).getNumberOfClusters();
			
			clusterRecall = numberOfUniqueCoveredClusters/numberOfTotalClusters;
			if(printDetails) System.err.println("Number of covered unique clusters:" + numberOfUniqueCoveredClusters + " for topic:"+ topicNr + " in run " +runName);
			if(printDetails) System.err.println("Number of total unique clusters:" + numberOfTotalClusters+ " for topic:"+ topicNr + " in run " +runName);
			if(printDetails) System.err.println("CLUSTER RECALL AT "+clusterRecallAtPosition + ": " + clusterRecall + " for topic:"+ topicNr + " in run " +runName);
			if(printDetails) System.err.println("----------");
		}
		else{
			System.err.println("Error CR: Requested topic >" + topicNr + "< could not be found in run : " +trc.getFilePath());
		}	
		return clusterRecall;
	}
	
	/** Return the cluster recall over all topics. Valid topic are the ones which are in the Qrel file.
	 * To calculate the average over the topics, the number of topics
	 * @param clusterRecallAtPosition
	 * @param trc
	 * @param qrelsContainer
	 * @param clusterAssessmentContainer
	 * @return
	 */
	public static double getAverageClusterRecall(int clusterRecallAtPosition, TrecResultsContainer trc, QrelsContainer qrelsContainer, HashMap<String, ClusterTopicAssessmentsContainer> clusterAssessmentContainer){
		double averageClusterRecall = 0.0d; 
		double sumOfClusterRecalls = 0;
		String[] topics = trc.getTopicNumbers(); // for all topic in result file
		for (String t : topics) {			
			if(qrelsContainer.isValidTopicNr(t)){
				double clusterRecall = getClusterRecall(t, clusterRecallAtPosition, trc, qrelsContainer, clusterAssessmentContainer);
				sumOfClusterRecalls += clusterRecall;
			}
			else{
				System.err.println("No valid topicNr : " + t + " in file " + trc.getFilePath());
			}
		}
		averageClusterRecall = sumOfClusterRecalls / trc.getNumberOfValidTopics(qrelsContainer); //divide through available topic in result file
		return averageClusterRecall;
	}

}